import pingen2sdk
import responses
import pytest


class TestWebhooks(object):
    @staticmethod
    def _construct_resource():
        access_token = "test_access_token"
        organisation_id = "testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"

        return pingen2sdk.Webhooks(organisation_id, access_token)

    @staticmethod
    def _get_expected_payload(webhook_id: str):
        return {
            "data": {
                "id": webhook_id,
                "type": "webhooks",
                "attributes": {
                    "event_category": "issues",
                    "url": "https://valid-url",
                    "signing_key": "d09a095a0d1d2ae896f985c0fff1ad51",
                },
                "relationships": {
                    "organisation": {
                        "links": {"related": "string"},
                        "data": {
                            "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                            "type": "organisations",
                        },
                    },
                },
                "links": {"self": "string"},
            },
            "included": [{}],
        }

    @responses.activate
    def test_get_details(self):
        webhook_id = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"
        url = "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/webhooks/%s" % (
            pingen2sdk.api_production,
            webhook_id,
        )

        webhooks = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json=self._get_expected_payload("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"),
            status=200,
        )

        response = webhooks.get_details(
            webhook_id, None, {"Content-Type": "application/vnd.api+json"}
        )

        assert response.data["data"]["id"] == webhook_id
        assert response.status_code == 200
        assert response.headers == {
            "Content-Type": "application/vnd.api+json",
            "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
        }

    @responses.activate
    def test_get_collection(self):
        url = (
            "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/webhooks"
            % pingen2sdk.api_production
        )

        webhooks = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json={
                "data": [
                    {
                        "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                        "type": "webhooks",
                        "attributes": {
                            "event_category": "issues",
                            "url": "https://valid-url",
                            "signing_key": "d09a095a0d1d2ae896f985c0fff1ad51",
                        },
                        "relationships": {
                            "organisation": {
                                "links": {"related": "string"},
                                "data": {
                                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                                    "type": "organisations",
                                },
                            },
                        },
                        "links": {"self": "string"},
                    }
                ],
                "included": [{}],
                "links": {
                    "first": "string",
                    "last": "string",
                    "prev": "string",
                    "next": "string",
                    "self": "string",
                },
                "meta": {
                    "current_page": 0,
                    "last_page": 0,
                    "per_page": 0,
                    "from": 0,
                    "to": 0,
                    "total": 0,
                },
            },
            status=200,
        )

        response = webhooks.get_collection()

        assert response.data["data"][0]["id"] == "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
        assert response.status_code == 200
        assert response.headers == {
            "Content-Type": "application/vnd.api+json",
            "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
        }

    @responses.activate
    def test_create(self):
        url = "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/webhooks" % (
            pingen2sdk.api_production,
        )

        webhooks = self._construct_resource()

        responses.post(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx2",
            },
            json=self._get_expected_payload("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxx11"),
            status=201,
        )

        response = webhooks.create(
            "issues",
            "https://valid-url",
            "d09a095a0d1d2ae896f985c0fff1ad51",
        )

        assert response.data["data"]["id"] == "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxx11"
        assert response.status_code == 201
        assert response.headers == {
            "Content-Type": "application/vnd.api+json",
            "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx2",
        }

    @responses.activate
    def test_delete(self):
        webhook_id = "testdelx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"
        url = "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/webhooks/%s" % (
            pingen2sdk.api_production,
            webhook_id,
        )

        webhooks = self._construct_resource()

        responses.delete(
            url,
            status=204,
        )

        response = webhooks.delete(
            webhook_id,
        )

        assert response.status_code == 204

    @responses.activate
    def test_delete_unauthorize(self):
        with pytest.raises(pingen2sdk.PingenError):
            webhook_id = "testdelx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"
            url = (
                "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/webhooks/%s"
                % (
                    pingen2sdk.api_production,
                    webhook_id,
                )
            )

            webhooks = self._construct_resource()

            responses.delete(
                url,
                json={
                    "errors": {
                        "code": "access_denied",
                        "title": "The resource owner or authorization server denied the request.",
                        "detail": "The JWT string must have two dots",
                        "source": {"pointer": None, "parameter": None},
                    }
                },
                status=401,
            )

            webhooks.delete(
                webhook_id,
            )



import hmac
import hashlib

from typing import Mapping

import pingen2sdk.response
from pingen2sdk import error


class IncomingWebhook(object):
    @staticmethod
    def construct_event(
        payload: str, header: Mapping[str, str], secret: str
    ) -> pingen2sdk.WebhookEvent:
        WebhookSignature.verify_header(payload, header, secret)

        return pingen2sdk.WebhookEvent(payload)


class WebhookSignature(object):
    @staticmethod
    def verify_header(payload: str, header: Mapping[str, str], secret: str) -> bool:
        signature = header.get("Signature")

        if not signature:
            raise error.WebhookSignatureException("Signature missing.")

        expected_sig = hmac.new(
            secret.encode("utf-8"),
            payload,
            hashlib.sha256,
        ).hexdigest()

        if signature != expected_sig:
            raise error.WebhookSignatureException("Webhook signature matching failed.")

        return True



import pingen2sdk
import json

from typing import Any, Mapping, Optional, List
from typing_extensions import Literal


class Batches(object):
    def __init__(
        self,
        organisation_id: str,
        access_token: str,
        use_staging: bool = False,
    ):
        self.organisation_id = organisation_id
        self.api_requestor = pingen2sdk.APIRequestor(access_token, use_staging)

    def get_details(
        self,
        batch_id: str,
        params: Optional[Mapping[str, Any]] = None,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_get_request(
            "/organisations/%s/batches/%s" % (self.organisation_id, batch_id),
            params,
            supplied_headers,
        )

    def get_collection(
        self,
        params: Optional[Mapping[str, Any]] = None,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_get_request(
            "/organisations/%s/batches" % self.organisation_id,
            params,
            supplied_headers,
        )

    def upload_and_create(
        self,
        path_to_file: str,
        name: str,
        icon: Literal[
            "campaign",
            "megaphone",
            "wave-hand",
            "flash",
            "rocket",
            "bell",
            "percent-tag",
            "percent-badge",
            "present",
            "receipt",
            "document",
            "information",
            "calendar",
            "newspaper",
            "crown",
            "virus",
        ],
        file_original_name: str,
        address_position: Literal["left", "right"],
        grouping_type: Literal["zip", "merge"],
        grouping_options_split_type: Literal["file", "page", "custom", "qr_invoice"],
        grouping_options_split_size: Optional[int] = None,
        grouping_options_split_separator: Optional[str] = None,
        grouping_options_split_position: Optional[
            Literal["first_page", "last_page"]
        ] = None,
    ) -> pingen2sdk.PingenResponse:
        file_upload = pingen2sdk.FileUpload(self.api_requestor)
        file_url, file_url_signature = file_upload.request_file_upload()
        file_upload.put_file(path_to_file, file_url)

        return self.create(
            file_url,
            file_url_signature,
            name,
            icon,
            file_original_name,
            address_position,
            grouping_type,
            grouping_options_split_type,
            grouping_options_split_size,
            grouping_options_split_separator,
            grouping_options_split_position,
        )

    def create(
        self,
        file_url: str,
        file_url_signature: str,
        name: str,
        icon: Literal[
            "campaign",
            "megaphone",
            "wave-hand",
            "flash",
            "rocket",
            "bell",
            "percent-tag",
            "percent-badge",
            "present",
            "receipt",
            "document",
            "information",
            "calendar",
            "newspaper",
            "crown",
            "virus",
        ],
        file_original_name: str,
        address_position: Literal["left", "right"],
        grouping_type: Literal["zip", "merge"],
        grouping_options_split_type: Literal["file", "page", "custom", "qr_invoice"],
        grouping_options_split_size: Optional[int] = None,
        grouping_options_split_separator: Optional[str] = None,
        grouping_options_split_position: Optional[
            Literal["first_page", "last_page"]
        ] = None,
    ) -> pingen2sdk.PingenResponse:
        attributes = {
            "file_url": file_url,
            "file_url_signature": file_url_signature,
            "name": name,
            "icon": icon,
            "file_original_name": file_original_name,
            "address_position": address_position,
            "grouping_type": grouping_type,
            "grouping_options_split_type": grouping_options_split_type,
        }

        if grouping_options_split_size is not None:
            attributes["grouping_options_split_size"] = grouping_options_split_size

        if grouping_options_split_separator is not None:
            attributes["grouping_options_split_separator"] = (
                grouping_options_split_separator
            )

        if grouping_options_split_position is not None:
            attributes["grouping_options_split_position"] = (
                grouping_options_split_position
            )

        return self.api_requestor.perform_post_request(
            "/organisations/%s/batches" % self.organisation_id,
            json.dumps({"data": {"type": "batches", "attributes": attributes}}),
        )

    def send(
        self,
        batch_id: str,
        delivery_product: Mapping[str, str],
        print_mode: str,
        print_spectrum: str,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_patch_request(
            "/organisations/%s/batches/%s/send" % (self.organisation_id, batch_id),
            json.dumps(
                {
                    "data": {
                        "id": batch_id,
                        "type": "batches",
                        "attributes": {
                            "delivery_products": delivery_product,
                            "print_mode": print_mode,
                            "print_spectrum": print_spectrum,
                        },
                    }
                }
            ),
        )

    def cancel(
        self,
        batch_id: str,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_cancel_request(
            "/organisations/%s/batches/%s/cancel" % (self.organisation_id, batch_id),
        )

    def delete(
        self,
        batch_id: str,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_delete_request(
            "/organisations/%s/batches/%s" % (self.organisation_id, batch_id),
        )

    def edit(
        self,
        batch_id: str,
        paper_types: List[str],
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_patch_request(
            "/organisations/%s/batches/%s" % (self.organisation_id, batch_id),
            json.dumps(
                {
                    "data": {
                        "id": batch_id,
                        "type": "batches",
                        "attributes": {
                            "paper_types": paper_types,
                        },
                    }
                }
            ),
        )

    def get_statistics(
        self,
        batch_id: str,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_get_request(
            "/organisations/%s/batches/%s/statistics" % (self.organisation_id, batch_id)
        )



import pingen2sdk
import responses


class TestUsers(object):
    @staticmethod
    def _construct_resource():
        access_token = "test_access_token"

        return pingen2sdk.UserAssociations(access_token, True)

    @responses.activate
    def test_get_details(self):
        user_id = "userxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx2"
        url = "%s/user/associations" % pingen2sdk.api_staging

        user_associations = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json={
                "data": [
                    {
                        "id": user_id,
                        "type": "associations",
                        "attributes": {
                            "role": "owner",
                            "status": "pending",
                            "created_at": "2020-11-19T09:42:48+0100",
                            "updated_at": "2020-11-19T09:42:48+0100",
                        },
                        "relationships": {
                            "organisation": {
                                "links": {"related": "string"},
                                "data": {
                                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                                    "type": "organisations",
                                },
                            }
                        },
                        "links": {"self": "string"},
                        "meta": {
                            "abilities": {
                                "self": {"join": "ok", "leave": "ok", "block": "ok"},
                                "organisation": {"manage": "ok"},
                            }
                        },
                    }
                ],
                "included": [{}],
                "links": {
                    "first": "string",
                    "last": "string",
                    "prev": "string",
                    "next": "string",
                    "self": "string",
                },
                "meta": {
                    "current_page": 0,
                    "last_page": 0,
                    "per_page": 0,
                    "from": 0,
                    "to": 0,
                    "total": 0,
                },
            },
            status=200,
        )

        response = user_associations.get_collection()

        assert response.data["data"][0]["id"] == user_id
        assert response.data["data"][0]["type"] == "associations"
        assert response.status_code == 200



import pingen2sdk
import requests
import pytest
import responses


class TestOAuth(object):
    def test_authorize_url_error(self):
        with pytest.raises(pingen2sdk.error.AuthenticationError):
            pingen2sdk.OAuth.authorize_url(
                scope="letter", state="RANDOMGENERATEDSTRING"
            )

    def test_authorize_url(self):
        url = pingen2sdk.OAuth.authorize_url(
            scope="letter",
            state="RANDOMGENERATEDSTRING",
            client_id="testClientId",
            response_type="code",
        )

        result = requests.utils.urlparse(url)
        query = result.query
        params = dict(x.split("=") for x in query.split("&"))

        assert result.scheme == "https"
        assert result.netloc == "identity.pingen.com"

        assert params["client_id"] == "testClientId"
        assert params["scope"] == "letter"
        assert params["state"] == "RANDOMGENERATEDSTRING"

    def test_authorize_url_staging(self):
        pingen2sdk.client_id = "testSetClientId"

        url = pingen2sdk.OAuth.authorize_url(
            use_staging=True, scope="letter", state="RANDOMGENERATEDSTRING"
        )

        result = requests.utils.urlparse(url)
        query = result.query
        params = dict(x.split("=") for x in query.split("&"))

        assert result.scheme == "https"
        assert result.netloc == "identity-staging.pingen.com"

        assert params["client_id"] == "testSetClientId"
        assert params["scope"] == "letter"
        assert params["state"] == "RANDOMGENERATEDSTRING"

    def test_token_error(self):
        with pytest.raises(pingen2sdk.error.AuthenticationError):
            pingen2sdk.client_id = "testSetClientId"

            pingen2sdk.OAuth.get_token(
                grant_type="client_credentials",
            )

    @responses.activate
    def test_token(self):
        pingen2sdk.client_id = "testSetClientId"

        responses.post(
            "https://api.pingen.com/auth/access-tokens",
            json={
                "token_type": "Bearer",
                "expires_in": 43200,
                "access_token": "YOUR_ACCESS_TOKEN",
            },
        )

        resp = pingen2sdk.OAuth.get_token(
            grant_type="client_credentials", client_secret="testClientSecret"
        )

        assert resp["access_token"] == "YOUR_ACCESS_TOKEN"

    @responses.activate
    def test_staging_token(self):
        pingen2sdk.client_id = "testSetClientId"
        pingen2sdk.client_secret = "testSetClientSecret"

        responses.post(
            "https://api-staging.pingen.com/auth/access-tokens",
            json={
                "token_type": "Bearer",
                "expires_in": 43200,
                "access_token": "YOUR_ACCESS_TOKEN",
            },
        )

        resp = pingen2sdk.OAuth.get_token(
            use_staging=True, grant_type="client_credentials"
        )

        assert resp["access_token"] == "YOUR_ACCESS_TOKEN"

    def test_get_token_from_implicit(self):
        resp = pingen2sdk.OAuth.get_token_from_implicit(
            "access_token=mock_access_token&token_type=Bearer&expires_in=43200&state=yourrandomstate"
        )

        assert resp["access_token"] == "mock_access_token"
        assert resp["expires_in"] == "43200"



import pingen2sdk
import responses


def _mock_empty_response():
    return {
        "data": [{}],
        "links": {
            "first": "string",
            "last": "string",
            "prev": "string",
            "next": "string",
            "self": "string",
        },
        "meta": {
            "current_page": 2,
            "last_page": 3,
            "per_page": 10,
            "from": 10,
            "to": 19,
            "total": 30,
        },
    }


class TestLetterEvents(object):
    @staticmethod
    def _construct_resource():
        access_token = "test_access_token"
        organisation_id = "testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"

        return pingen2sdk.LetterEvents(organisation_id, access_token)

    @responses.activate
    def test_get_collection(self):
        letter_id = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"
        url = (
            "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/letters/%s/events"
            % (
                pingen2sdk.api_production,
                letter_id,
            )
        )

        letters_events = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json=_mock_empty_response(),
            status=200,
        )

        response = letters_events.get_collection(
            letter_id, None, {"Content-Type": "application/vnd.api+json"}
        )

        assert len(response.data["data"][0]) == 0
        assert response.status_code == 200

    @responses.activate
    def test_get_issue_collection(self):
        url = (
            "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/letters/events/issues"
            % pingen2sdk.api_production
        )

        letters_events = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json=_mock_empty_response(),
            status=200,
        )

        response = letters_events.get_issue_collection()

        assert len(response.data["data"][0]) == 0
        assert response.status_code == 200

    @responses.activate
    def test_get_undeliverable_collection(self):
        url = (
            "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/letters/events/undeliverable"
            % pingen2sdk.api_production
        )

        letters_events = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json=_mock_empty_response(),
            status=200,
        )

        response = letters_events.get_undeliverable_collection()

        assert len(response.data["data"][0]) == 0
        assert response.status_code == 200

    @responses.activate
    def test_get_sent_collection(self):
        url = (
            "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/letters/events/sent"
            % pingen2sdk.api_production
        )

        letters_events = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json=_mock_empty_response(),
            status=200,
        )

        response = letters_events.get_sent_collection()

        assert len(response.data["data"][0]) == 0
        assert response.status_code == 200



import pingen2sdk
import responses


class TestFileUpload(object):
    @staticmethod
    def _construct_resource():
        access_token = "test_access_token"

        return pingen2sdk.FileUpload(pingen2sdk.APIRequestor(access_token))

    @responses.activate
    def test_request_file_upload(self):
        url = "%s/file-upload" % (pingen2sdk.api_production,)

        file_upload = self._construct_resource()

        expected_url = "https://s3.example/bucket/filename?signer=url"
        expected_signature = (
            "$2y$10$BLOzVbYTXrh4LZbSYNVf7eEDrc58vvQ9PRVZABqV/9WS1eqIcm3M"
        )

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json={
                "data": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "type": "file_uploads",
                    "attributes": {
                        "url": expected_url,
                        "url_signature": expected_signature,
                        "expires_at": "2020-11-19T09:42:48+0100",
                    },
                    "links": {"self": "string"},
                }
            },
        )

        url, url_signature = file_upload.request_file_upload()

        assert url == expected_url
        assert url_signature == expected_signature
